<?php
#
# file: phpini.inc
#
# Zend installation scripts library - php.ini routies
#  Copyright 2001 Zend Technologies Ltd. All Rights Reserved.
#




# return php.ini location
function guess_php_ini_location ()
{
  global $PREFIX;
  $PHP_INI = '';
  $locations = array ("$PREFIX/etc", '/usr/local/Zend/etc', '/usr/local/lib');
  foreach ($locations as $location) {
		if (is_file_silent ("$location/php.ini")) {
			$PHP_INI = "$location/php.ini";
			break;
		}
  }

  while (1) {
	$PHP_INI = inputbox ("Confirm the location of your php.ini file", $PHP_INI);
	if (!is_file_silent ($PHP_INI)) {
		msgbox("php.ini was not found at $PHP_INI. Please Retry.");
	}
	else
		break;
  }
  return ($PHP_INI);
} # end of guess_php_ini_location ()





# backup php.ini once
function backup_php_ini ()
{
  global $PHP_INI;

  if (! defined ('PHP_INI_BACKUPED'))
    {
      $backup = "$PHP_INI." . BACKUP_SUFFIX;
      $backup_took_place = 0;
      if (is_file_silent ($PHP_INI))
	{
	  copy_wrap ($PHP_INI, $backup);
	  $backup_took_place = 1;
	}
      elseif (is_link_silent ($PHP_INI))
	{
	  rename_wrap ($PHP_INI, $backup);
	  $backup_took_place = 1;
	}

      define ('PHP_INI_BACKUPED', 1);
      if ($backup_took_place == 1)
	{
	  msgbox ("Your old php.ini is saved as\n${backup}.");
	}
    }
} # end of backup_php_ini()





# arguments: commands for ini_modifier
function modify_php_ini ()
{
  global $TMPDIR, $PHP_INI;

  if (!isset ($PHP_INI) or !file_exists($PHP_INI)) {
      on_error ("The supplied location of php.ini (at '$PHP_INI') is incorrect");
  }
  $DIRECT_PHP_INI=strip_symlinks($PHP_INI);
  backup_php_ini();
  _fix_php_ini();

  # retrieve password from the user and place appropriate directive in php.ini

  $old_ini = file2str ($PHP_INI);
  $passwd = passwdbox();
  $md5_passwd = md5 ($passwd);
  $new_ini = preg_replace ('/(\s*zend_gui_password\s*=\s*)[a-zA-Z0-9]+/e', 
			   "'$1' . '$md5_passwd'", $old_ini);

  if ($old_ini == $new_ini) {
		if (! preg_match ('/\s*zend_gui_password\s*=\s*/', $old_ini)) {
			# no previous directive 'zend_gui_password' existed;
			# add one at the beginning of the file
			$new_ini = "zend_gui_password=${md5_passwd}\n\n" . $old_ini;
			fwrite_wrap ($PHP_INI, 'w', $new_ini);
		}
  }
  else
	  fwrite_wrap ($PHP_INI, 'w', $new_ini);


  $TMPFILE = "$TMPDIR/dialog";
  $ERRMSG = 'Error executing ini_modifier';
  $args = func_get_args();
  $fd = popen_wrap ("./ini_modifier -f $DIRECT_PHP_INI -n -q 2>$TMPFILE", $ERRMSG);
  if (fwrite ($fd, "$passwd\n" . join ("\n", $args) . "\nwrite\nquit\n") == -1) {
	  on_error ('Failed to modify '.$PHP_INI.' file (Error '.$ERRMSG.': '.file2str($TMPFILE).')');
  }
  if (($ret = (pclose ($fd)>>8)&0xFF) != 0) {
      if ($ret == 4) {
		  # 4 is the exit code ini_modifier returns
		  # when it fails to parse the php.ini file

		  $usrmsg = "There is an error in your existing PHP initialization file\n" .
					 "$PHP_INI\n" .
					 "The reported error was: " . file2str($TMPFILE) . "\n" .
					 "Please fix this error by editing the file manually and\n" .
					 "re-start the installer";
		  msgbox($usrmsg);
		  cleanup();
		  die();
      }
      else {
		  on_error ("$ERRMSG. Error code: $ret - ".file2str($TMPFILE));
	  }
  }
} # end of modify_php_ini()



function strip_symlinks($file)
{
  $new_file=$file;
  $count = 0;
  while (is_link_silent ($new_file))
    {
      if ($count++ >= 64)
	{
	  on_error ("Too many levels of symbolic links for $file");
	}
      $newer_file = readlink ($new_file);
      if (!$newer_file)
	{
	  on_error ("Unable to read symbolic link $new_file");
	}
      $new_file = $newer_file;
    }
  return $new_file;
}

# relocate php.ini to /usr/local/Zend/etc/, 
# making symlink from the former place
function relocate_php_ini()
{
  global $PHP_INI, $PREFIX;

  $new_place = "$PREFIX/etc";
  $new_ini   = "$new_place/php.ini";
  
  # find the real location of php.ini, in case of symlink(s)
  $old_ini = strip_symlinks($PHP_INI);
  $old_place = dirname($old_ini);

  if ($old_place != $new_place) 
  {
      if (is_file_silent ($new_ini))
	  {	
	  		rename_wrap ($new_ini, "${new_ini}." . BACKUP_SUFFIX . "1",
		       "Error saving previous copy of $new_ini");
	  }
      mkdir_silent($new_place);
      rename_wrap ($PHP_INI, "$new_ini", "Error moving php.ini to the $new_place directory");
      symlink ($new_ini, $PHP_INI) || on_error ("Error creating symbolic link $PHP_INI -> $new_ini");
      $PHP_INI = $new_ini;
      msgbox ("Your php.ini is relocated to the $new_place directory
and symbolic link from the former place $old_ini is created.");
  }
} # end of relocate_php_ini()





# put one zend_extension directive after another, if needed;
# return: 1 if php.ini was changed, 0 otherwise
function _swap_directives ($zend_module0, $zend_module1)
{
  global $php_ini_content, $zend_ext;

  if (
      (in_array ($zend_module0, array_keys ($zend_ext))) &&
      (in_array ($zend_module1, array_keys ($zend_ext)))
      ) {
      $line_num0 = $zend_ext[$zend_module0][1];
      $line_num1 = $zend_ext[$zend_module1][1];

      if ($line_num0 <= $line_num1) {
		  # remove $zend_module0 directive and append it next after $zend_module1
		  $save = $php_ini_content[$line_num0];

		  for ($i = $line_num0; $i < $line_num1; $i++)
			  $php_ini_content[$i] = $php_ini_content[$i+1];

		  $php_ini_content[$line_num1] = $save;
		  
		  $zend_ext[$zend_module0][1] = $line_num1;
		  $zend_ext[$zend_module1][1] = $line_num1 - 1;
	  
		  return (1);
	  }
      else
		  return (0);
  }
  else
	  return (0);

} # end of _swap_directives()





# rearrange loading order of two Zend products, if needed
# (correct order is: optimizer, accelerator, debugger)
# args: any distinct two of 'OPTIMIZER', 'CACHE' or 'DEBUGGER'
function reorder_php_ini ()
{
  # we assume that php.ini was located previously
  global $PHP_INI, $php_ini_content, $zend_ext;

  if (! is_file_silent ($PHP_INI))
    {
      on_error ("$PHP_INI not found");
    }

  # get line numbers of optimizer, accelerator, debugger directives
  $php_ini_content = file ($PHP_INI);
  $zend_ext = array();
  while (list ($line_num, $line) = each ($php_ini_content)) {
      $line = chop ($line);
      if (preg_match ('/^\s*zend_extension=[a-zA-Z0-9_\/\"]*Zend[a-zA-Z0-9_]+\.so/', $line)) {
		  $zend_module = basename ($line);
		  $zend_ext[$zend_module] = array ($line, $line_num);
	  }
  }
	 
  # place:
  # accelerator after optimizer,
  # debugger    after optimizer, and
  # debugger    after accelerator
  $modified0 = _swap_directives ('ZendAccelerator.so', 'ZendOptimizer.so');
  $modified1 = _swap_directives ('ZendDebugger.so',    'ZendOptimizer.so');
  $modified2 = _swap_directives ('ZendDebugger.so',    'ZendAccelerator.so');

  # if order was changed - back up php.ini
  if ($modified0 || $modified1 || $modified2)
    {
      backup_php_ini();
      fwrite_wrap ($PHP_INI, 'w', join ('', $php_ini_content));
    }
} # end of reorder_php_ini()




# perform various fixes of customer's php.ini
# (like fixing php distributions bugs) - this may change from time to time
function _fix_php_ini()
{
  global $PHP_INI;
  if (! defined ('PHP_INI_FIXED'))
    {
      backup_php_ini();
      $new_ini = preg_replace ('/\' at MYSQL_PORT/', '; at MYSQL_PORT', file2str ($PHP_INI));
      fwrite_wrap ($PHP_INI, 'w', $new_ini);
      define ('PHP_INI_FIXED', 1);
    }
} # end of fix_php_ini()


# end of phpini.inc
?>
